@@ -41,8 +41,8 @@ class Config(ConfigValue):
         self.zookeeper_tls_cert = None
         self.zookeeper_tls_key = None
         self.zookeeper_tls_ca = None
-        self.elementsdir = None
-        self.imagesdir = None
+        self.elements_dir = None
+        self.images_dir = None
         self.build_log_dir = None
         self.build_log_retention = None
         self.max_hold_age = None
@@ -55,8 +55,8 @@ class Config(ConfigValue):
                     self.providers == other.providers and
                     self.provider_managers == other.provider_managers and
                     self.zookeeper_servers == other.zookeeper_servers and
-                    self.elementsdir == other.elementsdir and
-                    self.imagesdir == other.imagesdir and
+                    self.elements_dir == other.elements_dir and
+                    self.images_dir == other.images_dir and
                     self.build_log_dir == other.build_log_dir and
                     self.build_log_retention == other.build_log_retention and
                     self.max_hold_age == other.max_hold_age and
@@ -64,10 +64,10 @@ class Config(ConfigValue):
         return False
 
     def setElementsDir(self, value):
-        self.elementsdir = value
+        self.elements_dir = value
 
     def setImagesDir(self, value):
-        self.imagesdir = value
+        self.images_dir = value
 
     def setBuildLog(self, directory, retention):
         if retention is None:
@@ -101,7 +101,7 @@ class Config(ConfigValue):
 
         for server in zk_cfg:
             z = zk.ZooKeeperConnectionConfig(server['host'],
-                                             server.get('port', 2181),
+                                             server.get('port', 2281),
                                              server.get('chroot', None))
             name = z.host + '_' + str(z.port)
             self.zookeeper_servers[name] = z
@@ -210,6 +210,7 @@ class DiskImage(ConfigValue):
         self.image_types = set([])
         self.pause = False
         self.python_path = 'auto'
+        self.shell_type = None
         self.rebuild_age = self.REBUILD_AGE
         self.release = ''
         self.username = 'zuul'
@@ -249,6 +250,9 @@ class DiskImage(ConfigValue):
         python_path = config.get('python-path', None)
         if python_path:
             self.python_path = python_path
+        shell_type = config.get('shell-type', None)
+        if shell_type:
+            self.shell_type = shell_type
         rebuild_age = config.get('rebuild-age', None)
         if rebuild_age:
             self.rebuild_age = rebuild_age
@@ -269,6 +273,7 @@ class DiskImage(ConfigValue):
                     other.image_types == self.image_types and
                     other.pause == self.pause and
                     other.python_path == self.python_path and
+                    other.shell_type == self.shell_type and
                     other.rebuild_age == self.rebuild_age and
                     other.release == self.release and
                     other.username == self.username)
@@ -358,7 +363,6 @@ def loadSecureConfig(config, secure_config_path, env=os.environ):
     if secure.get('zookeeper-servers', []):
         config.zookeeper_servers = {}
 
-    # TODO(Shrews): Support ZooKeeper auth
     config.setZooKeeperServers(secure.get('zookeeper-servers'))
     config.setSecureDiskimageEnv(
         secure.get('diskimages', []), secure_config_path)
