@@ -221,6 +221,19 @@ class Provider(ProviderNotifications):
         """
         pass
 
+    def startNodeCleanup(self, node):
+        '''Starts a background process to delete a node
+
+        This should return a NodeDeleter to implement and track the
+        deletion of a node.
+
+        :param Node node: A locked Node object representing the
+            instance to delete.
+
+        :returns: A NodeDeleter instance.
+        '''
+        pass
+
     @abc.abstractmethod
     def cleanupNode(self, node_id):
         """Cleanup a node after use
@@ -552,8 +565,8 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
             declined_reasons.append('it would exceed quota')
 
         if declined_reasons:
-            self.log.debug("Declining node request because %s",
-                           ', '.join(declined_reasons))
+            self.log.info("Declining node request because %s",
+                          ', '.join(declined_reasons))
             self.decline_request()
             self._declinedHandlerCleanup()
             return
@@ -607,12 +620,13 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
                 try:
                     self.zk.storeNode(node)
                 except Exception:
-                    self.log.exception("Error clearing node allocation:")
+                    self.log.exception(
+                        "Error clearing node allocation of node %s:", node.id)
 
             try:
                 self.zk.unlockNode(node)
             except Exception:
-                self.log.exception("Error unlocking node:")
+                self.log.exception("Error unlocking node %s:", node.id)
             self.log.debug("Unlocked node %s", node.id)
 
         self.nodeset = []
@@ -797,7 +811,7 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
     def launchesComplete(self):
         '''
         Handler needs to implement this to check if all nodes in self.nodeset
-        have completed the launch sequence..
+        have completed the launch sequence.
 
         This method will be called periodically to check on launch progress.
 
@@ -808,13 +822,30 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
 
 
 class ConfigValue(object, metaclass=abc.ABCMeta):
-    @abc.abstractmethod
+    ignore_equality = []
+
     def __eq__(self, other):
-        pass
+        if isinstance(other, self.__class__):
+            for k in set(list(self.__dict__.keys()) +
+                         list(other.__dict__.keys())):
+                if k in self.ignore_equality:
+                    continue
+                if self.__dict__.get(k) != other.__dict__.get(k):
+                    return False
+            return True
+        return False
+
+    def __hash__(self):
+        return hash(frozenset(self.__dict__))
 
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def __repr__(self):
+        if hasattr(self, 'name'):
+            return "<%s %s>" % (self.__class__.__name__, self.name)
+        return "<%s>" % (self.__class__.__name__,)
+
 
 class ConfigPool(ConfigValue, metaclass=abc.ABCMeta):
     '''
@@ -826,13 +857,6 @@ class ConfigPool(ConfigValue, metaclass=abc.ABCMeta):
         self.max_servers = math.inf
         self.node_attributes = None
 
-    def __eq__(self, other):
-        if isinstance(other, ConfigPool):
-            return (self.labels == other.labels and
-                    self.max_servers == other.max_servers and
-                    self.node_attributes == other.node_attributes)
-        return False
-
     @classmethod
     def getCommonSchemaDict(self):
         '''
@@ -872,11 +896,6 @@ class DriverConfig(ConfigValue):
     def __init__(self):
         self.name = None
 
-    def __eq__(self, other):
-        if isinstance(other, DriverConfig):
-            return self.name == other.name
-        return False
-
 
 class ProviderConfig(ConfigValue, metaclass=abc.ABCMeta):
     """The Provider config interface
@@ -891,17 +910,6 @@ class ProviderConfig(ConfigValue, metaclass=abc.ABCMeta):
         self.driver.name = provider.get('driver', 'openstack')
         self.max_concurrency = provider.get('max-concurrency', -1)
 
-    def __eq__(self, other):
-        if isinstance(other, ProviderConfig):
-            return (self.name == other.name and
-                    self.provider == other.provider and
-                    self.driver == other.driver and
-                    self.max_concurrency == other.max_concurrency)
-        return False
-
-    def __repr__(self):
-        return "<Provider %s>" % self.name
-
     @classmethod
     def getCommonSchemaDict(self):
         return {
