@@ -29,16 +29,6 @@ class ProviderCloudImage(ConfigValue):
         self.connection_type = None
         self.connection_port = None
 
-    def __eq__(self, other):
-        if isinstance(other, ProviderCloudImage):
-            return (self.name == other.name
-                    and self.image_id == other.image_id
-                    and self.username == other.username
-                    and self.python_path == other.python_path
-                    and self.connection_type == other.connection_type
-                    and self.connection_port == other.connection_port)
-        return False
-
     def __repr__(self):
         return "<ProviderCloudImage %s>" % self.name
 
@@ -49,6 +39,8 @@ class ProviderCloudImage(ConfigValue):
 
 
 class ProviderLabel(ConfigValue):
+    ignore_equality = ['pool']
+
     def __init__(self):
         self.name = None
         self.cloud_image = None
@@ -63,27 +55,13 @@ class ProviderLabel(ConfigValue):
         self.pool = None
         self.tags = None
 
-    def __eq__(self, other):
-        if isinstance(other, ProviderLabel):
-            # NOTE(Shrews): We intentionally do not compare 'pool' here
-            # since this causes recursive checks with ProviderPool.
-            return (other.name == self.name
-                    and other.cloud_image == self.cloud_image
-                    and other.ebs_optimized == self.ebs_optimized
-                    and other.instance_type == self.instance_type
-                    and other.key_name == self.key_name
-                    and other.volume_size == self.volume_size
-                    and other.volume_type == self.volume_type
-                    and other.userdata == self.userdata
-                    and other.iam_instance_profile == self.iam_instance_profile
-                    and other.tags == self.tags)
-        return False
-
     def __repr__(self):
         return "<ProviderLabel %s>" % self.name
 
 
 class ProviderPool(ConfigPool):
+    ignore_equality = ['provider']
+
     def __init__(self):
         self.name = None
         self.max_cores = None
@@ -142,19 +120,6 @@ class ProviderPool(ConfigPool):
             ]
             full_config.labels[label['name']].pools.append(self)
 
-    def __eq__(self, other):
-        if isinstance(other, ProviderPool):
-            # NOTE(Shrews): We intentionally do not compare 'provider' here
-            # since this causes recursive checks with OpenStackProviderConfig.
-            return (super().__eq__(other)
-                    and other.name == self.name
-                    and other.subnet_id == self.subnet_id
-                    and other.security_group_id == self.security_group_id
-                    and other.public_ip == self.public_ip
-                    and other.host_key_checking == self.host_key_checking
-                    and other.labels == self.labels)
-        return False
-
     def __repr__(self):
         return "<ProviderPool %s>" % self.name
 
@@ -170,17 +135,6 @@ class AwsProviderConfig(ProviderConfig):
         self.cloud_images = {}
         super().__init__(provider)
 
-    def __eq__(self, other):
-        if isinstance(other, AwsProviderConfig):
-            return (super().__eq__(other)
-                    and other.profile_name == self.profile_name
-                    and other.region_name == self.region_name
-                    and other.pools == self.pools
-                    and other.boot_timeout == self.boot_timeout
-                    and other.launch_retries == self.launch_retries
-                    and other.cloud_images == self.cloud_images)
-        return False
-
     @property
     def pools(self):
         return self.__pools
@@ -225,6 +179,7 @@ class AwsProviderConfig(ProviderConfig):
 
             i.username = image.get('username', None)
             i.python_path = image.get('python-path', 'auto')
+            i.shell_type = image.get('shell-type', None)
             i.connection_type = image.get('connection-type', 'ssh')
             i.connection_port = image.get(
                 'connection-port',
@@ -272,6 +227,7 @@ class AwsProviderConfig(ProviderConfig):
             'name': str,
             'connection-type': str,
             'connection-port': int,
+            'shell-type': str,
             'image-id': str,
             "image-filters": [image_filters],
             'username': str,
