@@ -111,6 +111,7 @@ class AwsInstanceLauncher(NodeLauncher):
         self.node.host_keys = keys
         self.node.username = self.label.cloud_image.username
         self.node.python_path = self.label.cloud_image.python_path
+        self.node.shell_type = self.label.cloud_image.shell_type
         self.zk.storeNode(self.node)
         self.log.info("Instance %s is ready", instance_id)
 
@@ -127,7 +128,7 @@ class AwsNodeRequestHandler(NodeRequestHandler):
     def alive_thread_count(self):
         count = 0
         for t in self._threads:
-            if t.isAlive():
+            if t.is_alive():
                 count += 1
         return count
 
