@@ -16,11 +16,13 @@
 
 import logging
 import math
+import operator
 import os
 import os.path
 import socket
 import threading
 import time
+import uuid
 
 from kazoo import exceptions as kze
 
@@ -45,73 +47,6 @@ LOCK_CLEANUP = 8 * HOURS
 SUSPEND_WAIT_TIME = 30
 
 
-class NodeDeleter(threading.Thread):
-    log = logging.getLogger("nodepool.NodeDeleter")
-
-    def __init__(self, zk, provider_manager, node):
-        threading.Thread.__init__(self, name='NodeDeleter for %s %s' %
-                                  (node.provider, node.external_id))
-        self._zk = zk
-        self._provider_manager = provider_manager
-        self._node = node
-
-    @staticmethod
-    def delete(zk_conn, manager, node, node_exists=True):
-        '''
-        Delete a server instance and ZooKeeper node.
-
-        This is a class method so we can support instantaneous deletes.
-
-        :param ZooKeeper zk_conn: A ZooKeeper object to use.
-        :param ProviderManager provider_manager: ProviderManager object to
-            use fo deleting the server.
-        :param Node node: A locked Node object that describes the server to
-            delete.
-        :param bool node_exists: True if the node actually exists in ZooKeeper.
-            An artifical Node object can be passed that can be used to delete
-            a leaked instance.
-        '''
-        try:
-            node.state = zk.DELETING
-            zk_conn.storeNode(node)
-            if node.external_id:
-                manager.cleanupNode(node.external_id)
-                manager.waitForNodeCleanup(node.external_id)
-        except exceptions.NotFound:
-            NodeDeleter.log.info("Instance %s not found in provider %s",
-                                 node.external_id, node.provider)
-        except Exception:
-            NodeDeleter.log.exception(
-                "Exception deleting instance %s from %s:",
-                node.external_id, node.provider)
-            # Don't delete the ZK node in this case, but do unlock it
-            if node_exists:
-                zk_conn.unlockNode(node)
-            return
-
-        if node_exists:
-            NodeDeleter.log.info(
-                "Deleting ZK node id=%s, state=%s, external_id=%s",
-                node.id, node.state, node.external_id)
-            # This also effectively releases the lock
-            zk_conn.deleteNode(node)
-            manager.nodeDeletedNotification(node)
-
-    def run(self):
-        # Since leaked instances won't have an actual node in ZooKeeper,
-        # we need to check 'id' to see if this is an artificial Node.
-        if self._node.id is None:
-            node_exists = False
-        else:
-            node_exists = True
-
-        try:
-            self.delete(self._zk, self._provider_manager,
-                        self._node, node_exists)
-        except Exception:
-            self.log.exception("Error deleting node %s:", self._node)
-
-
 class PoolWorker(threading.Thread, stats.StatsReporter):
     '''
     Class that manages node requests for a single provider pool.
@@ -137,9 +72,9 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
         self.request_handlers = []
         self.watermark_sleep = nodepool.watermark_sleep
         self.zk = self.getZK()
-        self.launcher_id = "%s-%s-%s" % (socket.gethostname(),
-                                         os.getpid(),
-                                         self.name)
+        self.launcher_id = "%s-%s-%s" % (socket.getfqdn(),
+                                         self.name,
+                                         uuid.uuid4().hex)
         stats.StatsReporter.__init__(self)
 
     # ---------------------------------------------------------------
@@ -173,14 +108,8 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
         # which express a preference for a specific provider.
         launchers = self.zk.getRegisteredLaunchers()
 
-        # Sort requests by queue priority, then, for all requests at
-        # the same priority, use the relative_priority field to
-        # further sort, then finally, the submission order.
-        requests = list(self.zk.nodeRequestIterator())
-        requests.sort(key=lambda r: (r.id.split('-')[0],
-                                     r.relative_priority,
-                                     r.id.split('-')[1]))
-
+        requests = sorted(self.zk.nodeRequestIterator(),
+                          key=operator.attrgetter("priority"))
         for req in requests:
             if not self.running:
                 return True
@@ -278,6 +207,7 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
             except kze.SessionExpiredError:
                 # If we lost our ZooKeeper session, we've lost our NodeRequest
                 # lock so it's no longer active
+                log.error("Removing request handler (lost zookeeper session)")
                 continue
             except Exception:
                 # If we fail to poll a request handler log it but move on
@@ -697,11 +627,8 @@ class DeletedNodeWorker(BaseCleanupWorker):
         self.log.info("Deleting %s instance %s from %s",
                       node.state, node.external_id, node.provider)
         try:
-            t = NodeDeleter(
-                self._nodepool.getZK(),
-                self._nodepool.getProviderManager(node.provider),
-                node)
-            t.start()
+            pm = self._nodepool.getProviderManager(node.provider)
+            pm.startNodeCleanup(node)
         except Exception:
             self.log.exception("Could not delete instance %s on provider %s",
                                node.external_id, node.provider)
@@ -891,7 +818,7 @@ class NodePool(threading.Thread):
         self._stop_event.set()
         # Our run method can start new threads, so make sure it has
         # completed before we continue the shutdown.
-        if self.isAlive():
+        if self.is_alive():
             self.join()
 
         if self._cleanup_thread:
@@ -910,7 +837,7 @@ class NodePool(threading.Thread):
         # terminated.
         self.log.debug("Stopping pool threads")
         for thd in self._pool_threads.values():
-            if thd.isAlive():
+            if thd.is_alive():
                 thd.stop()
             self.log.debug("Waiting for %s" % thd.name)
             thd.join()
@@ -1157,7 +1084,7 @@ class NodePool(threading.Thread):
                             self.log.info("Starting %s" % t.name)
                             t.start()
                             self._pool_threads[key] = t
-                        elif not self._pool_threads[key].isAlive():
+                        elif not self._pool_threads[key].is_alive():
                             self._pool_threads[key].join()
                             t = PoolWorker(self, provider.name, pool.name)
                             self.log.info("Restarting %s" % t.name)
