@@ -25,6 +25,7 @@ import boto3
 from moto import mock_ec2
 import yaml
 
+from nodepool import config as nodepool_config
 from nodepool import tests
 from nodepool import zk
 from nodepool.nodeutils import iterate_timeout
@@ -49,7 +50,8 @@ class TestDriverAws(tests.DBTestCase):
                           host_key_checking=True,
                           userdata=None,
                           public_ip=True,
-                          tags=[]):
+                          tags=[],
+                          shell_type=None):
         aws_id = 'AK000000000000000000'
         aws_key = '0123456789abcdef0123456789abcdef0123456789abcdef'
         self.useFixture(
@@ -80,6 +82,11 @@ class TestDriverAws(tests.DBTestCase):
             'port': self.zookeeper_port,
             'chroot': self.zookeeper_chroot,
         }
+        raw_config['zookeeper-tls'] = {
+            'ca': self.zookeeper_ca,
+            'cert': self.zookeeper_cert,
+            'key': self.zookeeper_key,
+        }
         raw_config['providers'][0]['pools'][0]['subnet-id'] = subnet_id
         raw_config['providers'][0]['pools'][0]['security-group-id'] = sg_id
         raw_config['providers'][0]['pools'][1]['subnet-id'] = subnet_id
@@ -92,6 +99,8 @@ class TestDriverAws(tests.DBTestCase):
         raw_config['providers'][0]['pools'][4]['security-group-id'] = sg_id
         raw_config['providers'][0]['pools'][5]['subnet-id'] = subnet_id
         raw_config['providers'][0]['pools'][5]['security-group-id'] = sg_id
+        raw_config['providers'][0]['pools'][6]['subnet-id'] = subnet_id
+        raw_config['providers'][0]['pools'][6]['security-group-id'] = sg_id
 
         with tempfile.NamedTemporaryFile() as tf:
             tf.write(yaml.safe_dump(
@@ -147,6 +156,7 @@ class TestDriverAws(tests.DBTestCase):
                 self.assertEqual(node.connection_type, 'ssh')
                 self.assertEqual(node.attributes,
                                  {'key1': 'value1', 'key2': 'value2'})
+                self.assertEqual(node.shell_type, shell_type)
                 if host_key_checking:
                     nodescan.assert_called_with(
                         node.interface_ip,
@@ -246,3 +256,14 @@ class TestDriverAws(tests.DBTestCase):
                                tags=[
                                    {"Key": "Name", "Value": "different-name"}
                                ])
+
+    def test_ec2_machine_shell_type(self):
+        self._test_ec2_machine('ubuntu1404-with-shell-type',
+                               shell_type="csh")
+
+    def test_aws_config(self):
+        configfile = self.setup_config('aws-config.yaml')
+        config = nodepool_config.loadConfig(configfile)
+        self.assertIn('ec2-us-west-2', config.providers)
+        config2 = nodepool_config.loadConfig(configfile)
+        self.assertEqual(config, config2)
