@@ -19,7 +19,6 @@ import time
 
 from nodepool import tests
 from nodepool import zk
-from nodepool.driver.kubernetes import provider
 
 
 class FakeCoreClient(object):
@@ -44,7 +43,7 @@ class FakeCoreClient(object):
         self.namespaces.append(FakeNamespace)
         return FakeNamespace
 
-    def delete_namespace(self, name, delete_body):
+    def delete_namespace(self, name, body):
         to_delete = None
         for namespace in self.namespaces:
             if namespace.metadata.name == name:
@@ -95,13 +94,12 @@ class TestDriverKubernetes(tests.DBTestCase):
         self.fake_k8s_client = FakeCoreClient()
         self.fake_rbac_client = FakeRbacClient()
 
-        def fake_get_client(*args):
-            return self.fake_k8s_client, self.fake_rbac_client
+        def fake_get_client(log, context, ctor=None):
+            return None, None, self.fake_k8s_client, self.fake_rbac_client
 
-        self.useFixture(fixtures.MockPatchObject(
-            provider.KubernetesProvider, '_get_client',
-            fake_get_client
-        ))
+        self.useFixture(fixtures.MockPatch(
+            'nodepool.driver.kubernetes.provider.get_client',
+            fake_get_client))
 
     def test_kubernetes_machine(self):
         configfile = self.setup_config('kubernetes.yaml')
