@@ -18,43 +18,77 @@ import logging
 
 from nodepool import tests
 from nodepool import zk
-from nodepool.driver.openshift import provider
+
+
+class FakeOpenshiftProjectsQuery:
+
+    def __init__(self, client):
+        self.client = client
+
+    def get(self):
+        class FakeProjectsResult:
+            def __init__(self, items):
+                self.items = items
+
+        return FakeProjectsResult(self.client.projects)
+
+    def delete(self, name):
+        to_delete = None
+        for project in self.client.projects:
+            if project.metadata.name == name:
+                to_delete = project
+                break
+        if not to_delete:
+            raise RuntimeError("Unknown project %s" % name)
+        self.client.projects.remove(to_delete)
+
+
+class FakeOpenshiftProjectRequestQuery:
+
+    def __init__(self, client):
+        self.client = client
+
+    def create(self, body):
+        class FakeProject:
+            class metadata:
+                name = body['metadata']['name']
+        self.client.projects.append(FakeProject)
+        return FakeProject
+
+
+class FakeOpenshiftRoleBindingQuery:
+
+    def __init__(self, client):
+        self.client = client
+
+    def create(self, body, namespace):
+        return
+
+
+class FakeOpenshiftResources:
+
+    def __init__(self, client):
+        self.client = client
+
+    def get(self, api_version=None, kind=None):
+        if kind == 'Project':
+            return FakeOpenshiftProjectsQuery(self.client)
+        if kind == 'ProjectRequest':
+            return FakeOpenshiftProjectRequestQuery(self.client)
+        if kind == 'RoleBinding':
+            return FakeOpenshiftRoleBindingQuery(self.client)
+        raise NotImplementedError
 
 
 class FakeOpenshiftClient(object):
     def __init__(self):
         self.projects = []
 
-        class FakeApi:
-            class configuration:
-                host = "http://localhost:8080"
-                verify_ssl = False
-        self.api_client = FakeApi()
-
-    def list_project(self):
-        class FakeProjects:
-            items = self.projects
-        return FakeProjects
-
-    def create_project_request(self, proj_body):
-        class FakeProject:
-            class metadata:
-                name = proj_body['metadata']['name']
-        self.projects.append(FakeProject)
-        return FakeProject
-
-    def delete_project(self, name):
-        to_delete = None
-        for project in self.projects:
-            if project.metadata.name == name:
-                to_delete = project
-                break
-        if not to_delete:
-            raise RuntimeError("Unknown project %s" % name)
-        self.projects.remove(to_delete)
-
-    def create_namespaced_role_binding(self, ns, role_binding_body):
-        return
+        class FakeConfiguration:
+            host = "http://localhost:8080"
+            verify_ssl = False
+        self.configuration = FakeConfiguration()
+        self.resources = FakeOpenshiftResources(self)
 
 
 class FakeCoreClient(object):
@@ -91,13 +125,12 @@ class TestDriverOpenshift(tests.DBTestCase):
         self.fake_os_client = FakeOpenshiftClient()
         self.fake_k8s_client = FakeCoreClient()
 
-        def fake_get_client(*args):
-            return self.fake_os_client, self.fake_k8s_client
+        def fake_get_client(log, context, ctor=None):
+            return None, None, self.fake_k8s_client, self.fake_os_client
 
-        self.useFixture(fixtures.MockPatchObject(
-            provider.OpenshiftProvider, '_get_client',
-            fake_get_client
-        ))
+        self.useFixture(fixtures.MockPatch(
+            'nodepool.driver.openshift.provider.get_client',
+            fake_get_client))
 
     def test_openshift_machine(self):
         configfile = self.setup_config('openshift.yaml')
@@ -120,6 +153,7 @@ class TestDriverOpenshift(tests.DBTestCase):
         self.assertEqual(node.connection_type, 'kubectl')
         self.assertEqual(node.connection_port.get('token'), 'fake-token')
         self.assertEqual(node.python_path, '/usr/bin/python3')
+        self.assertEqual(node.shell_type, 'csh')
         self.assertEqual(node.attributes,
                          {'key1': 'value1', 'key2': 'value2'})
 
@@ -148,8 +182,13 @@ class TestDriverOpenshift(tests.DBTestCase):
         self.assertIsNotNone(node.launcher)
         self.assertEqual(node.connection_type, 'project')
         self.assertEqual(node.connection_port.get('token'), 'fake-token')
+        self.assertEqual(node.python_path, 'auto')
+        self.assertIsNone(node.shell_type)
 
         node.state = zk.DELETING
         self.zk.storeNode(node)
 
         self.waitForNodeDeletion(node)
+
+        self.assertEqual(len(self.fake_os_client.projects), 0,
+                         'Project must be cleaned up')
